module fileutil.file;

private import std.file;
private import std.string;
private import std.regexp;
private import std.path;
private import nscr.plugin;
private import nscr.result;
private import nscr.charset;
private import fileutil.meddle;

/// ディレクトリを作成する。
/**
	@param path 作成するディレクトリのパス
	@retval false ファイルが既にある場合
	@retval true  成功
**/
bool mkdir(dirpath path){
	if(std.file.exists(path)){
		return false;
	}
	std.file.mkdir(path);
	return true;
}


/// ディレクトリを削除する。
/**
	@param path 削除するディレクトリのパス
	@retval false ファイルがない場合
	@retval true  成功
**/
bool rmdir(dirpath path){
	if(!std.file.exists(path)){
		return false;
	}
	std.file.rmdir(path);
	return true;
}


/// 現在の作業ディレクトリを取得する。
/**
	@return 現在の作業ディレクトリ
	@note D言語的には'.'を指す名前だが、
	      NScripterとしてしっくりくるように命名。
**/
char[] curdir(){
	return std.file.getcwd();
}


/// ファイルを読み込む。
/**
	@param path 読み込むファイルのパス
	@return ファイルの内容
	@note NScripterでは改行を判定できないため、
	      改行文字に変換しておく。
**/
char[] fileread(filepath path){
	return std.string.join(
		std.string.splitlines(
			nscr.charset.fromNScrCharset(cast(char[])std.file.read(path))
		),
		nscr.plugin.linesep
	);
}


/// ファイルを書き込む。
/**
	@param path 書き込むファイルのパス
	@param text 書き込む内容
	@return true
	@note 例外発生時のみfalseを返す。
**/
bool filewrite(filepath path, nscrtext text){
	std.file.write(path, text);
	return true;
}


/// ファイルに追記する。
/**
	@param path 書き込むファイルのパス
	@param text 書き込む内容
	@return true
	@note 例外発生時のみfalseを返す。
**/
bool fileappend(filepath path, nscrtext text){
	std.file.append(path, text);
	return true;
}


/// ファイルに1行追記する。
/**
	@param path 書き込むファイルのパス
	@param text 書き込む内容
	@return true
	@note 例外発生時のみfalseを返す。
**/
bool fileappendln(filepath path, nscrtext text){
	std.file.append(path, cast(char[])text ~ "\r\n");
	return true;
}


/// ファイル名を変更する。
/**
	@param from 変更前のファイルのパス
	@param to   変更後のファイルのパス
	@retval false 変更前のファイルなしor変更後のファイルあり
	@retval true  変更成功
**/
bool filerename(filepath from, filepath to){
	if(!std.file.exists(from)){
		return false;
	}
	if(std.file.exists(to)){
		return false;
	}
	std.file.rename(from, to);
	return true;
}


/// ファイルサイズを取得する。
/**
	@param path ファイルのパス
	@retval -1  ファイルなし
	@retval >=0 ファイルサイズ
**/
int filesize(filepath path){
	if(!std.file.exists(path)){
		return -1;
	}
	return cast(int)std.file.getSize(path);
}


/// ファイルを複製する。
/**
	@param from 複製元のファイルのパス
	@param to   複製先のファイルのパス
	@retval false 複製元のファイルなし
	@retval true  複製成功
**/
bool filecopy(filepath from, filepath to){
	if(!std.file.exists(from)){
		return false;
	}
	std.file.copy(from, to);
	return true;
}


/// ファイル一覧を取得する。
/**
	@param path 取得対象のディレクトリのパス
	@param pattern ファイル名のパターン
	@return intval: ファイル数, strval: ファイル一覧
**/
Result filelist(dirpath path, char[] pattern){
	assert(path != null);

	if(pattern == null || pattern.length == 0){
		pattern = ".*";
	}
	char[][] result;
	RegExp filter = RegExp(pattern);

	bool listing(char[] filename)
	{
	    filename = std.path.join(path, filename);
		if(std.file.isfile(filename) && filter.test(filename)){
		    if(std.string.find(filename, curdir()) != -1){
		        result ~= filename[curdir().length+1..$]; // カレントディレクトリ\を除外
		    }
		}
		return true; // continue
	}

	std.file.listdir(path, &listing);
	return Result(result.length, std.string.join(result, nscr.plugin.listsep));
}


/// ディレクトリ一覧を取得する。
/**
	@param path 取得対象のディレクトリのパス
	@return intval: ディレクトリ数, strval: ディレクトリ一覧
**/
Result dirlist(dirpath path){
	assert(path != null);

	char[][] result;

	bool listing(char[] dirname)
	{
	    dirname = std.path.join(path, dirname);
		if(std.file.isdir(dirname)){
		    if(std.string.find(dirname, curdir()) != -1){
			    result ~= dirname[curdir().length+1..$]; // カレントディレクトリ\を除外
		    }
		}
		return true; // continue
	}

	std.file.listdir(path, &listing);
	return Result(result.length, std.string.join(result, nscr.plugin.listsep));
}
